#include "testlib.h"
#include <vector>

using namespace std;

typedef long long int64;

inline int64 gcd(int64 a, int64 b) {
    while (b > 0) {
        int64 y = a % b;
        a = b, b = y;
    }
    return a;
}

struct rational {
	int64 unit, p, q;

	rational(int64 p, int64 q, int64 unit = 0) {
		this->unit = unit;
		this->p = p;
		this->q = q;
		if (this->q < 0) {
			this->p = -this->p;
			this->q = -this->q;
		}
		int64 g = gcd(abs(this->p), abs(this->q));
		this->p /= g, this->q /= g;
		int64 division = this->p / this->q;
		this->p -= division * this->q;
		this->unit += division;
		while (this->p < 0) {
			this->p += this->q;
			this->unit--;
		}
	}
};

rational operator + (const rational &a, const rational &b) {
	return rational(a.p * b.q + a.q * b.p, a.q * b.q, a.unit + b.unit);
}

rational operator * (const rational &a, const int64 &b) {
	return rational(a.p * b, a.q, a.unit * b);
}

bool operator < (const rational &a, const rational &b) {
	if (a.unit != b.unit) {
		return a.unit < b.unit;
	}
	return a.p * b.q < a.q * b.p;
}

rational abs(rational x) {
    if (x.unit < 0) {
        return x * (-1);
    }
    return x;
}

rational getDistance(pair<rational, rational> point, tuple<int, int, int> line) {
	int64 a, b, c; tie(a, b, c) = line;
	return abs(point.first * a + point.second * b + rational(c, 1));
}

pair<rational, rational> readAns(InStream &in, int n, const vector<tuple<int, int, int>> &lines) {
	int f = in.readInt(1, n) - 1, s = in.readInt(1, n) - 1;
	int64 a1, b1, c1, a2, b2, c2;
	tie(a1, b1, c1) = lines[f], c1 = -c1;
	tie(a2, b2, c2) = lines[s], c2 = -c2;
	int64 d = a1 * b2 - a2 * b1;
	if (d == 0) {
		in.quitf(_wa, "too bad: lines %d and %d do not intersect :(", f + 1, s + 1);
	}
	int64 dx = c1 * b2 - c2 * b1, dy = a1 * c2 - a2 * c1;
	return make_pair(rational(dx, d), rational(dy, d));
}

int main(int argc, char **argv) {
	registerTestlibCmd(argc, argv);
	int n = inf.readInt(), t = inf.readInt();
	vector<tuple<int, int, int>> lines, routes;
	for (int i = 0; i < n; i++) {
		int a = inf.readInt(), b = inf.readInt(), c = inf.readInt();
		lines.emplace_back(a, b, c);
	}
	for (int i = 0; i < t; i++) {
		int u = inf.readInt(), v = inf.readInt(), w = inf.readInt();
		routes.emplace_back(u, v, w);
	}
	for (int i = 0; i < t; i++) {
		pair<rational, rational> contestantIntersection = readAns(ouf, n, lines);
		pair<rational, rational> juryIntersection = readAns(ans, n, lines);
		rational contestantDistance = getDistance(contestantIntersection, routes[i]);
		rational juryDistance = getDistance(juryIntersection, routes[i]);
		if (contestantDistance < juryDistance) {
			quitf(_fail, "contestant is better than jury :(");
		}
		if (juryDistance < contestantDistance) {
			quitf(_wa, "the distance is too large on %d-th request: %lld + %lld / %lld against %lld + %lld / %lld :(", 
				i + 1, contestantDistance.unit, contestantDistance.p, contestantDistance.q, juryDistance.unit, juryDistance.p, juryDistance.q);
		}
	}
	quitf(_ok, "all answers are awesome and correct :)");
}