#include <cstdio>
#include <algorithm>
#include <vector>
#include <cassert>

using namespace std;

const double eps = 1e-9;

struct point {
  double x, y;
  point(){}
  point(double x, double y):x(x), y(y){}
  bool operator<(const point& a) const {
    return y < a.y - 1e-9;
  }
};

struct line {
  int a, b, c;
  line() {}
  line(int a, int b, int c):a(a), b(b), c(c) {}
  void read() {
    scanf("%d%d%d",&a,&b,&c);
  }
  double dist(const point& p) const {
    return abs(a * p.x + b * p.y + c);
  }
};

point intersect(const line &l1, const line &l2) {
  double d = l1.a * l2.b - l1.b * l2.a;
  assert(fabs(d) > eps);
  point res(
    (l1.b * 1.0 * l2.c - l1.c * 1.0 * l2.b) / d,
    (l1.a * 1.0 * l2.c - l1.c * 1.0 * l2.a) / -d
  );
  assert(l1.dist(res) < 1e-1);
  assert(l2.dist(res) < 1e-1);
  return res;
}

bool parallel(const line &l1, const line &l2) {
  return l1.a * l2.b - l1.b * l2.a == 0; 
}

vector<line> all;
int ba, bb;

void check(const line& to, int pa, int pb) {
  if (parallel(all[pa], all[pb])) return;
  if (ba == -1) {
    ba = pa;
    bb = pb;
  }
  point p1 = intersect(all[ba], all[bb]);
  point p2 = intersect(all[pa], all[pb]);
  if (to.dist(p1) > to.dist(p2)) {
    ba = pa;
    bb = pb;
  }
}

void solve(const line &to) {
  vector<int> p;
  vector<int> np;

  for (int i = 0; i < (int)all.size(); i++) {
    if (parallel(all[i], to)) {
      p.push_back(i);
    } else {
      np.push_back(i);    
    }
  }

  sort(np.begin(), np.end(), [&to](int a, int b) {
        return intersect(all[a], to) < intersect(all[b], to);
      });

  assert(np.size() > 0);
  ba = bb = -1;

  for (int i : p) check(to, i, np[0]);
  for (int i = 1; i < (int)np.size(); i++)
      check(to, np[i-1], np[i]);

  printf("%.20lf\n", to.dist(intersect(all[ba], all[bb])) / sqrt(to.a * to.a + to.b * to.b));
}


int main() {
  int n, m;
  scanf("%d%d",&n,&m);
  all.resize(n);
  for (int i = 0; i < n; i++)
    all[i].read();

  for (int i = 0; i < m; i++) {
    line l;
    l.read();
    solve(l);
  }
}
